<?php
/* $Id: class_album.inc,v 1.19.2.7 2006/12/06 05:21:26 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the album.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

function class_album_register() {
    $class = new stdClass();
    $class->class = 'album';
    $class->max_size = 0;
    $class->mime_ext = Array();
    $class->create = '_class_album_create';
    $class->update = '_class_album_update';
    $class->load = '_class_album_load';
    $class->form = '_class_album_form';
    $class->destroy = '_class_album_destroy';
    $class->access = 'create acidfree albums';
    $class->weight = 0;
    return $class;
}

function _class_album_create(&$node, &$file) {
    $node->thumb = '';
    $node->weight -= 30;
    $extras = array(
            'sort' => $node->sort,
        );
    $node->small = serialize($extras);
}

function _class_album_update(&$node) {
    if ($node->thumb == '0') {
        $node->thumb = '';
    }
    $node->weight -= 30;
    $extras = array(
            'sort' => $node->sort,
        );
    $extras = serialize($extras);
    $per_class = array(
        'names' => array(
            "share=%d",
            "weight=%d",
            "small='%s'",
            "thumb='%s'",
        ),
        'values' => array(
            $node->share,
            $node->weight,
            $extras,
            $node->thumb,
        ),
    );
    return $per_class;
}

function _class_album_load(&$node, &$additions) {
    $extras = unserialize($additions->small);
    if (!is_array($extras))
        $extras = array();
    return $extras;
}

function _class_album_form($op, &$node) {
    $form = array();
    $form['share'] = array(
            '#type' => 'checkbox',
            '#title' => t("Allow others to post items to this album"),
            '#default_value' => $node->share,
        );
    switch ($op) {
    case 'new':
        break;
    case 'preview':
        $mythumb = (is_object($node->thumb) ? $node->thumb->fid : $node->thumb);
        $form['thumb'] = array(
                '#type' => 'hidden',
                '#value' => ($mythumb?$mythumb:'0'),
            );
        break;
    case 'update':
        $mythumb = (is_object($node->thumb) ? $node->thumb->fid : $node->thumb);
        $options['0'] = 'Random';
        $children = _acidfree_get_children($node, -1, 0, "class <> 'album'");
        if (!is_array($children)) {
            if (count($children) == 1)
                $children = Array($children);
            else
                $children = Array();
        }
        foreach ($children as $child) {
            $thumb = (is_object($child->thumb) ? $child->thumb->fid : $child->thumb);
            if (strlen($child->title) > 20)
                $title = substr($child->title, 0, 20)."...";
            else
                $title = $child->title;
            $options[$thumb] = $title;
        }
        $form['thumb'] = array(
                '#type' => 'select',
                '#title' => t("Album thumbnail"),
                '#default_value' => ($mythumb?$mythumb:'0'),
                '#options' => $options,
                '#attributes' => array('onchange' => "set_thumb(this, {$node->nid}, ".count($children).");"),
                '#acidfree_form' => true,
            );
        $sort_options = array(
                'global' => t('Use global setting'),
                'iage' => t('Most recent first'),
                'age' => t('Chronologically'),
                'alpha' => t('Alphabetically'),
            );
        $form['sort'] = array(
                '#type' => 'select',
                '#title' => t('Sort this album by'),
                '#default_value' => $node->sort,
                '#options' => $sort_options,
            );
        break;
    }
    return $form;
}

function _class_album_destroy(&$node) {
    // delete all children
    $kids = _acidfree_get_children($node);
    foreach ($kids as $kid) {
        if ($kid->class != 'album' && count($kid->parent) > 1) {
            // delete from this album only
            $key = array_search($node->nid, $kid->parent);
            unset($n->parent[$key]);
            node_save($kid);
        } else {
            node_delete($kid->nid);
        }
    }
}

function _album_create_links(&$node) {
    global $acidfree_types;
    $links = Array();
    $show_box = node_access('update', $node) ||
        (_acidfree_can_create() && $node->share) ||
        user_access('can upload to any album');
    if (!$show_box)
        return $links;
    foreach ($acidfree_types as $type) {
        if (user_access($type->access)){
          if(preg_match('/^[aeiou]/',$type->class)){ 
            $links[] = l(t('Add an %type', array('%type' => $type->class)),
                    "node/add/acidfree/{$type->class}/{$node->nid}");
          }
          else
          {
            $links[] = l(t('Add a %type', array('%type' => $type->class)),
                    "node/add/acidfree/{$type->class}/{$node->nid}");            
          }
        }
    }
    if (user_access('acidfree mass import'))
        $links[] = l(t('Mass import'), "node/add/acidfree/mass/{$node->nid}");
    return theme('item_list',$links);
}

function _acidfree_get_album_thumb(&$node) {
    $query = "SELECT n.nid, child FROM {node} n ".
      "INNER JOIN {acidfree} ON n.nid = {acidfree}.aid ".
      "INNER JOIN {acidfree_hierarchy} ON {acidfree}.aid = {acidfree_hierarchy}.child ".
      "WHERE {acidfree_hierarchy}.parent = '%s' AND {acidfree}.class %s 'album' ".
      "ORDER BY RAND() LIMIT 1";
    $result = db_query(db_rewrite_sql($query), $node->nid, '<>');

    if (db_num_rows($result) != 1) {
        // Re-use query from above except change operation from <> to =
        $result = db_query(db_rewrite_sql($query), $node->nid, '=');
        if (db_num_rows($result) != 1)
            return '';
        $row = db_fetch_array($result);
        return _acidfree_get_album_thumb(acidfree_get_node_by_id($row[child]));
    } else {
        $thumb = db_fetch_array($result);
        $thumb = acidfree_get_node_by_id($thumb['child']);
        $thumb = _acidfree_get_thumb_url($thumb);
    }   
    return $thumb;
}

function theme_acidfree_print_full_album($node) {
    $ncols = variable_get('acidfree_cols', 5);
    $nrows = variable_get('acidfree_rows', 3);
    $nodes_per_page = $ncols*$nrows;
    $nodes = _acidfree_get_children($node, $nodes_per_page, ALBUM_PAGER);
    if (count($nodes) == 0) 
        return "This album is empty<br>";
    $output .= theme('pager', NULL, $nodes_per_page, ALBUM_PAGER);
    foreach ($nodes as $child_node) {
        $row[] = Array(
                'data' => theme("acidfree_print_thumb_{$child_node->class}",
                                $child_node, $node),
                'class' => 'album-cell');
        if (++$count % $ncols == 0) {
            $rows[] = Array('data' => $row, 'class' => 'none');
            unset($row);
        }
    }
    if (isset($row))
        $rows[] = Array('data' => $row, "class" => "none");
    $output .= theme('table', NULL, $rows, Array('id' => 'acidfree-main'), false);
    $output .= "<div class='acidfree-body'>{$node->body}</div>";
    return $output;
}

function theme_acidfree_print_thumb_album(&$node) {
    $album_link = "node/{$node->nid}";
    $image = _acidfree_get_thumb_url($node);
    if ($image != '') {
        $image = l(theme('image', $image, $node->title, $node->title, 
                    array(
                        'class'=>'album-thumb',
                        'id'=>"acidfree-thumb{$node->nid}"),
                    false),
                $album_link, array(), NULL, NULL, true, true);
    }
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $output .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $output .= "<div class='folder' onclick='document.location=\"".url($album_link, null, null, true)."\"'>";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $output .= "<span></span>\n";
    $output .= $image;
    $output .= '</div><div>'.$node->title;
    //$output .= '</div><div>'.l($node->title, $album_link);
    $output .= "</div></div>\n";
    return $output;
}

?>
